
/*========================================================================================================================*/
/*                                                                                                                        */
/*                                              A C T I V A T E D   C O R E                                               */
/*                                                                                                                        */
/*   Activated Core is a game which intends to mix several game genres, including platforming, action, puzzle and more.   */
/*   The first version was a tile based platformer only, which then expanded beyond the tiles with large bitmapped maps.  */
/*     This allowed for fast rendering which was needed back then. The game was originally programmed in Visual Basic,    */
/*   using the extraordinary powers of DirectDraw 4 in the DirectX library (chuckles). This was year 2004. By the means   */
/*   of switching permanently to Linux, the development of this version stopped, until May 20:th 2010. The game is now    */
/*                              beging completely rewritten in C++ using OpenGL  and GLUT.                                */
/*                                                                                                                        */
/*                                                                                                                        */
/*                                                                                                                        */
/*                                                B A S I C   S T O R Y                                                   */
/*                                                                                                                        */
/*   Our character (nameless at the moment) was given a mission to explore a distant planet. On her way back to earth,    */
/*   she discovered at a distance that the earch atmosphere now have a purple hue. As she is getting closer and closer    */
/*   she picks up a transmission from the headquarters. The distress call mentioning something about foreign lifeforms    */
/*    converting earth into a machine, before it vanishes. When entering through the atmosphere, the ship breaks down     */
/*    and crashes. Altough she survives, she is weak and most of the equipment was destroyed in the crash. What is she    */
/*                                          going to do, to save mankind?                                                 */
/*                                                                                                                        */
/*========================================================================================================================*/

/*========================================================================================================================*/
/*                                                                                                                        */
/*   This game (including all game content, excluding FMOD [fmod.org] and TGALoader.h) was created by André Lundkvist     */
/*                                             during the years 2004-2011.                                                */
/*                                                                                                                        */
/*   Considering the effort, if you feel that the code has been useful for any of your projects, or if you just want      */
/*                 to say hello or give feedback, don't hesitate to contact me at ( admin@jiisuki.net ).                  */
/*                                 Please enter ACTIVATED CORE as the mail subject!                                       */
/*                                                                                                                        */
/*========================================================================================================================*/

/*========================================================================================================================*/
/*                                                                                                                        */
/*                                                  D I S C L A M E R                                                     */
/*                                                                                                                        */
/*    This game and code is provided as is, the creator holds no responsibility for any crashes or destruction of data    */
/*                                  that occurs, because this should never happen anyway :P                               */
/*                                                                                                                        */
/*                         J U S T   G O   A N D   P L A Y   T H E   G A M E   A L R E A D Y !                            */
/*                                                                                                                        */
/*========================================================================================================================*/



/* main.cpp
 *
 * This file contains all needed class headers, main function and a few more.
 * The game is built on c++ classes, which depends on each other in a nasty way ;)
 * The dependencies are such that the include order in this file need to be the way
 * they are at the moment. I will consider changing the structure later on
 *
 */

/* Header files */
#include <GL/glut.h>             // GLUT library
#include <GL/gl.h>               // OpenGL32 library
#include <GL/glu.h>              // GLu32 library
#include <stdio.h>               // Standard file i/o
#include <stdlib.h>              // For malloc/free
#include <string.h>              // For string operations
#include <math.h>                // For math operations
#include <unistd.h>              // For sleep operations (get control of that fps)
#include <time.h>                // For time
#include "include/global.h"      // For global settings of the game. cls_AC
#include "include/graphics.h"    // The cls_GfxFcns class (graphics functions, draw surface etc). Also holds variable for GLUT window
#include "include/gui.h"         // The cls_GUI class (Graphical User Interface), including console system
#include "include/sfx.h"         // The cls_SoundEngine class. Uses FMOD to play Sound and sound effects
#include "include/controls.h"    // The cls_Control class. Holds settings for the controls
#include "include/menu.h"        // The cls_Menu class. Holds the structure of all menus
#include "include/map.h"         // The cls_Map class. Loads and stores map information and data (textures, items etc)
#include "include/player.h"      // The cls_Character class. This contains functions for the player (fire, jump, animation, etc)
#include "include/starfield.h"   // The cls_Starfield class. As the name suggests, class for the background starfield
#include "include/font.h"        // The cls_Font class. All functions needed for writing text on the screen with this game
#include "include/configfiles.h" // The cls_ConfigurationFile class. Has functions to save and load profile settings
#include "include/intro.h"       // Header file for the intro sequence
#include "include/terminals.h"   // Header file for terminal contents (All valid Terminal ID used in single player)



/* LoadGLTextures()
 * Function to load all textures needed at the beginning of the game. */
void LoadGLTextures(){

  // Load fonts used by GUI and Menus.
  SYSTEM.SysMessage("Building Font 1");
  Font.BuildFont(0);
  SYSTEM.SysMessage("Building Font 2");
  Font.BuildFont(1);
  SYSTEM.SysMessage("Building Font 3");
  Font.BuildFont(2);

};


/* SetupDefaultGUI()
 * Function to set up all GUI rectangles, colors etc. */
void SetupDefaultGUI(){
  SYSTEM.SysMessage("Console");
  GUI.ClearConsole();
}

/* SetupGameSettings()
 * This function prints some information and sets up some non-profile specific settings of the game */
void SetupGameSettings(){
  // Inform that the game is starting and print out when the game was compiled.
  SYSTEM.NewSection("Starting Activated Core");
  printf("File was %s\n",AC.GetCompiled());

  // Display loaded control bindings.
  SYSTEM.NewSection("Loading Control Scheme");
  Control.CheckBinds();

  // Setup default menu selection = Introduction
  SYSTEM.NewSection("Selecting Menu");
  Menu.SetMenuSelection(MENU_INTRO);
  Menu.SetItemSelection(0);

  // Load default GUI setup.
  SYSTEM.NewSection("Loading GUI Scheme");
  SetupDefaultGUI();

  // Load textures (build fonts etc)
  SYSTEM.NewSection("Loading Graphics");
  LoadGLTextures();

  // Code the introduction screen welcome text.
  SYSTEM.NewSection("Coding Text Segments");
  SYSTEM.SysMessage("Main Menu");
  Menu.lenwelc = Font.ScrambleText(40,1,"A C T I V A T E D   C O R E",&Menu.welc[0],&Menu.scrm[0]);
  // Code all introduction credits.
  SYSTEM.SysMessage("Introduction");
  InitIntro();

  // Start the sound system.
  SYSTEM.NewSection("Initializing Sound");
  Sound.PlayTune(MUSIC_TUNE_INTRO);
  printf("Playing.\n");

  // Game should now start...
  SYSTEM.NewSection("Activated Core Running");
  
}




/* QuitAC()
 * Function to handle the shutdown, freeing of memory resources, deleting textures etc. */
void QuitAC(){

  // Indicate new section and save profile settings.
  SYSTEM.NewSection("Powering Down Activated Core");
  SYSTEM.SysMessage("Saving Profile Settings");
  Profile.SaveProfileSettings();

  // Set no key repeat ignore. (default)
  SYSTEM.SysMessage("Restoring Keyboard");
  glutIgnoreKeyRepeat(0);

  // Kill window.
  SYSTEM.SysMessage("Destroying Window");
  glutDestroyWindow(Glut_Window);

  // Close FSound.
  SYSTEM.SysMessage("Stopping Sound");
  Sound.Close();

  // Unload any loaded map.
  SYSTEM.SysMessage("Destroying Textures");
  if( Map.GetLoaded() )
    Map.Unload();

  // Destroy font textures.
  SYSTEM.SysMessage("Destroying Fonts");
  int i;
  for( i = 0; i < NumFonts; i++ ){
    glDeleteTextures(1, Font.GetTex(i));
  }

  // Destroy player textures.
  Player.KillTexture();

  // Shutdown pretty much completed.
  SYSTEM.NewSection("Power Down Completed!");
  SYSTEM.SysMessage("Thank you for playing!");
  SYSTEM.SysMessage("");
  SYSTEM.SysMessage("Make sure to check out the website:");
  SYSTEM.SysMessage(AC.GetWebsite());

  // Exit the program. Normal termination.
  exit(0);
}

/* GameControl(unsigned char key)
 * Handles the key bindings when in the actual platforming gameplay. */
void GameControl(unsigned char key){
  if( key == Control.GetBoundKey(CTRL_KEY_ESCAPE) ){
    AC.SpecifyLoop(LOOP_MAIN_MENU);
    Sound.PlayTune(MUSIC_TUNE_INTRO);
    Map.Unload();
    Menu.SetMenuSelection(MENU_MAIN);
  }
  if( key == Control.GetBoundKey(CTRL_KEY_UP) ){
    Player.ActionEnterTerminal();
  }
  if( key == Control.GetBoundKey(CTRL_KEY_DOWN) ){
    Player.Crouch(1);
  }
  if( key == Control.GetBoundKey(CTRL_KEY_LEFT) ){
    Player.Move(CHAR_MOVE_LEFT);
  }
  if( key == Control.GetBoundKey(CTRL_KEY_RIGHT) ){
    Player.Move(CHAR_MOVE_RIGHT);
  }
  if( key == Control.GetBoundKey(CTRL_KEY_TOGGLE_CONSOLE) ){
    Player.ActionEnterTerminal();
  }
  if( key == Control.GetBoundKey(CTRL_KEY_ACTION_1) ){
    Player.Fire();
  }
  if( key == Control.GetBoundKey(CTRL_KEY_ACTION_2) ){
    Player.Jump();
  }
  if( key == Control.GetBoundKey(CTRL_KEY_TEMPORARY_DEBUG_USAGE) ){
    Player.SpawnPlayer(Map.GetSpawnX(0),Map.GetSpawnY(0));
  }
}

/* MenuInGameControl(unsigned char key)
 * Function to handle all in game menu control. Basically the same as the main menu,
 * but with fewer options and some things to handle resuming etc. */
void MenuInGameControl(unsigned char key){
if( key == Control.GetBoundKey(CTRL_KEY_ESCAPE) )
  AC.Quit();
}

/* TerminalControl(unsigned char key)
 * Function to handle the terminal part of the game. Has special features to translate
 * keyascii into alpha-numeric letters as commands. */
void TerminalControl(unsigned char key){
  char tmp[CONSOLE_LINE_WIDTH] = {0};

  if( key == Control.GetBoundKey(CTRL_KEY_ESCAPE) ){
    Player.ActionLeaveTerminal();
  } else if( key == 8 ){ // Backspace
    strcpy(tmp,GUI.GetCurrentConsoleCommand());
    tmp[strlen(tmp)-1] = '\0';
    GUI.SetCurrentConsoleCommand(tmp);
  } else if( key == Control.GetBoundKey(CTRL_KEY_CONFIRM_OPTION) ){
    CheckTerminalCommand(Player.GetActiveTerminalID(),GUI.GetCurrentConsoleCommand());
  } else {
    if( isalnum(key) || key == 32 ){
      strcpy(tmp,GUI.GetCurrentConsoleCommand());
      tmp[strlen(tmp)] = key;
      GUI.SetCurrentConsoleCommand(tmp);
    }
  }

}

/* MenuControl(unsigned char key)
 * Function to handle control of the main menu. */
void MenuControl(unsigned char key){
    char tmp[64] = {0};

  /* If escape is pressed, kill everything. */
  if( key == Control.GetBoundKey(CTRL_KEY_ESCAPE) ){
    switch( Menu.GetMenu() ){
      case MENU_MAIN:
        AC.Quit();
        break;
      case MENU_SETTINGS_MAIN:
        Menu.SetMenuSelection(MENU_MAIN);
        break;
      case MENU_SETTINGS_PROFILE:
        Menu.SetMenuSelection(MENU_SETTINGS_MAIN);
        break;
      case MENU_SETTINGS_CONTROLS:
        Menu.SetMenuSelection(MENU_SETTINGS_MAIN);
        break;
      case MENU_SETTINGS_CHECK_BINDS:
        Menu.SetMenuSelection(MENU_SETTINGS_CONTROLS);
        break;
      case MENU_SETTINGS_SET_BINDS:
        Menu.SetMenuSelection(MENU_SETTINGS_CONTROLS);
        break;
      case MENU_SETTINGS_SFX:
        if( Sound.GetTune() != MUSIC_TUNE_INTRO )
          Sound.PlayTune(MUSIC_TUNE_INTRO);
        Menu.SetMenuSelection(MENU_SETTINGS_MAIN);
        break;
      case MENU_SETTINGS_GFX:
        Menu.SetMenuSelection(MENU_SETTINGS_MAIN);
        break;
      default:
        Menu.SetMenuSelection(MENU_MAIN);
        break;
    }
  }

  if( key == Control.GetBoundKey(CTRL_KEY_UP) ){
    if( Menu.GetMenu() != MENU_LOADING_MAP ) {
      Menu.MenuUp();
    }
  }
  if( key == Control.GetBoundKey(CTRL_KEY_DOWN) ){
    if( Menu.GetMenu() != MENU_LOADING_MAP ) {
      Menu.MenuDown();
    }
  }
  if( key == Control.GetBoundKey(CTRL_KEY_LEFT) ){
    switch( Menu.GetMenu() ){
      case MENU_SETTINGS_SFX:
        Sound.PlayPrev();
        sprintf(tmp,"Playing: %s",Sound.GetTuneName());
        Menu.SetOptionalText(tmp);
        break;
      default:
        break;
    }
  }
  if( key == Control.GetBoundKey(CTRL_KEY_RIGHT) ){
    switch( Menu.GetMenu() ){
      case MENU_SETTINGS_SFX:
        Sound.PlayNext();
        sprintf(tmp,"Playing: %s",Sound.GetTuneName());
        Menu.SetOptionalText(tmp);
        break;
      default:
        break;
    }
  }
  if( key == Control.GetBoundKey(CTRL_KEY_ACTION_1) ){
    if( Menu.GetMenu() == MENU_SETTINGS_PROFILE )
      Player.Fire();
  }
  if( key == Control.GetBoundKey(CTRL_KEY_ACTION_2) ){
    if( strcmp(Menu.GetItemName(0),"Quit") == 0 ){
      AC.Quit();
    }
  }
  if( key == Control.GetBoundKey(CTRL_KEY_CONFIRM_OPTION) ){
    switch( Menu.GetMenu() ){
      case MENU_INTRO:
        Menu.SetMenuSelection(MENU_MAIN);
        break;

      case MENU_LOADING_MAP:
        break;

      case MENU_MAIN:
        switch( Menu.GetItem() ){
          case 0:
            SYSTEM.NewSection("Loading Map"); // Create new logging section.
            Player.BuildAnimations(); // Load player textures (should only be done for the first mapload at the session)
            Map.LoadMap(AC.GetStoryMap()); // Load map file.
            Player.SpawnPlayer(Map.GetSpawnX(0),Map.GetSpawnY(0)); // Set player starting positions.
            AC.SpecifyLoop(LOOP_GAME); // Graphics Loop.
            break;
          case 1:
            Menu.SetMenuSelection(MENU_SETTINGS_MAIN);
            break;
          case 2:
            InitIntro();
            Menu.SetMenuSelection(MENU_INTRO);
            break;
          case 3:
            AC.Quit();
            break;
          default:
            break;
        }
        break;

      case MENU_SETTINGS_MAIN:
        switch( Menu.GetItem() ){
          case 0:
            Menu.SetMenuSelection(MENU_SETTINGS_PROFILE);
            break;
          case 1:
            Menu.SetMenuSelection(MENU_SETTINGS_CONTROLS);
            break;
          case 2:
            Menu.SetMenuSelection(MENU_SETTINGS_SFX);
            break;
          case 3:
            Menu.SetMenuSelection(MENU_SETTINGS_GFX);
            break;
          default:
            break;
        }
        break;

      case MENU_SETTINGS_PROFILE:
//        switch( Menu.GetItem() ){
//          case 0:
//            Menu.SetMenuSelection(MENU_SETTINGS_PROFILE);
//            break;
//          case 1:
//            Menu.SetMenuSelection(MENU_SETTINGS_CONTROLS);
//            break;
//          case 2:
//            Menu.SetMenuSelection(MENU_SETTINGS_SFX);
//            break;
//          case 3:
//            Menu.SetMenuSelection(MENU_SETTINGS_GFX);
//            break;
//          default:
//            break;
//        }
        break;

        case MENU_SETTINGS_CONTROLS:
        switch( Menu.GetItem() ){
          case 0:
            Menu.SetMenuSelection(MENU_SETTINGS_CHECK_BINDS);
            break;
          case 1:
            Menu.SetMenuSelection(MENU_SETTINGS_SET_BINDS);
            break;
          default:
            break;
        }
        break;

        case MENU_SETTINGS_SFX:
        switch( Menu.GetItem() ){
          case 0:
            Menu.SetMenuSelection(MENU_SETTINGS_MAIN);
            break;
          default:
            break;
        }
        break;

        case MENU_SETTINGS_GFX:
        switch( Menu.GetItem() ){
          case 0:
            if( AC.GetFullscreen() ){
              AC.SetWindowed();
              Menu.SetOptionalText("Fullscreen Disabled (Please Restart)");
            } else {
              AC.SetFullscreen();
              Menu.SetOptionalText("Fullscreen Enabled (Please Restart)");
            }
            break;
          case 1:
            if( AC.GetResolution(0) >= AC.GetResolution(1) ){
              AC.SetResolution(0, AC.GetAspectRatio(0) );
            } else {
              AC.SetResolution(AC.GetResolution(0)+1,AC.GetAspectRatio(0));
            }
            Menu.SetOptionalText(AC.GetResolutionDesc());
            break;
          case 2:
            if( AC.GetAspectRatio(0) >= AC.GetAspectRatio(1) ){
              AC.SetResolution(AC.GetResolution(0), 0 );
            } else {
              AC.SetResolution(AC.GetResolution(0),AC.GetAspectRatio(0)+1);
            }
            Menu.SetOptionalText(AC.GetResolutionDesc());
            break;
          default:
            break;
        }
        break;

      default:
        break;
    }

  }
  
}

/* The function called whenever a key is released. */
void keyReleased(unsigned char key, int x, int y){
  if( AC.GetSpecifiedLoop() == LOOP_GAME ){
    if( key == Control.GetBoundKey(CTRL_KEY_LEFT) ){
      Player.Move(CHAR_MOVE_STOP);
    }
    if( key == Control.GetBoundKey(CTRL_KEY_RIGHT) ){
      Player.Move(CHAR_MOVE_STOP);
    }
    if( key == Control.GetBoundKey(CTRL_KEY_DOWN) ){
      Player.Crouch(0);
    }
  }
}

/* The function called whenever a key is pressed. */
void keyPressed(unsigned char key, int x, int y)
{
  /* avoid thrashing this procedure */
  usleep(100);

  switch( AC.GetSpecifiedLoop() ){
    case LOOP_MAIN_MENU:
      MenuControl(key);
      break;
    case LOOP_GAME:
      GameControl(key);
      break;
    case LOOP_IN_GAME_MENU:
      MenuInGameControl(key);
      break;
    case LOOP_TERMINAL:
      TerminalControl(key);
      break;
    default:
      MenuControl(key);
      break;
  }

}

/* DrawMainGame()
 * This function draws the frames of the main platforming gameplay.
 * Including drawing of map, player, interface etc. */
void DrawMainGame(){
  time_t start,end;
  time(&start);

//  Map.SetLightSources(1);
  GFX.SetLightning(1);
    Map.DrawMap(AC.GetHalfW()-Player.GetPosX()-16,AC.GetHalfH()-Player.GetPosY()-16);
    Map.DrawMapObjects(Player.GetPosX(),Player.GetPosY());
    Player.DrawPlayer();
  GFX.SetLightning(0);

  // Print On Screen Message
  Font.glPrint(AC.GetHalfW()-strlen(GUI.ReadOSDMessage())*12/2,AC.GetHalfH()+24,GUI.ReadOSDMessage(),1);

  char tmph[64] = {0};
  char tmpa[64] = {0};
  char tmpb[64] = {0};

  sprintf(tmph,"Health:  %i",Player.GetHealth());
  sprintf(tmpa,"Armor:   %i",Player.GetArmor());
  sprintf(tmpb,"Battery: %i",Player.GetBattery());

  Font.glPrint(16,16,tmph,1);
  Font.glPrint(16,16+24,tmpa,1);
  Font.glPrint(16,16+48,tmpb,1);




  // Update some Physics
  Player.PerformPhysics();
  Player.StepAnimationFrame(0);

  

  usleep(AC.GetProgramHoldTime());

  // Get FPS
  AC.FPSc++;
  time(&end);
  AC.dif = AC.dif + difftime (end,start);
  if( AC.dif >= 1 ){
    sprintf(AC.FPS,"Running this screen at %i FPS",AC.FPSc);
    AC.FPSc = 0;
    AC.dif = 0;
    Menu.ShowInfo = !Menu.ShowInfo;
    Menu.InfoDelay++;
    if( Menu.InfoDelay > Menu.InfoDelayStop ){
      Menu.InfoDelay = 0;
      Starfield.SetFacing(int(rand()%360));
    }
  }
}

void DrawTerminal(){
  time_t start,end;
  time(&start);

  int i; 

  Font.glPrint(16,16,Map.GetTerminalLine(Player.GetActiveTerminalID(),0),1);

  for( i = 0; i < CONSOLE_NUMBER_OF_LINES; i++ ){
    Font.glPrint(16,48+16*i,GUI.GetConsoleLine(i),2*GUI.IsConsoleLineSystem(i));
  }

  Font.glPrint(16,AC.GetScreenHeight()-48,">> ",2);
  Font.glPrint(52,AC.GetScreenHeight()-48,GUI.GetCurrentConsoleCommand(),0);

  usleep(AC.GetProgramHoldTime());

  // Get FPS
  AC.FPSc++;
  time(&end);
  AC.dif = AC.dif + difftime (end,start);
  if( AC.dif >= 1 ){
    sprintf(AC.FPS,"Running this screen at %i FPS",AC.FPSc);
    AC.FPSc = 0;
    AC.dif = 0;
    Menu.ShowInfo = !Menu.ShowInfo;
    Menu.InfoDelay++;
    if( Menu.InfoDelay > Menu.InfoDelayStop ){
      Menu.InfoDelay = 0;
      Starfield.SetFacing(int(rand()%360));
    }
  }
}




void DrawMenu(){

  time_t start,end;
  time(&start);

  bool HoldASec[2] = {0};
  int i;

  switch( Menu.GetMenu() ){
    case MENU_INTRO:

      GFX.SetLightning(1);
      for( i = 0; i < 2; i++ ){
        if( IntroL[IntroCounter+i].Decode < IntroL[IntroCounter+i].LenText ){
          if( IntroL[IntroCounter+i].ScrambleData[IntroL[IntroCounter+i].Decode] > 0 ){
            IntroL[IntroCounter+i].Text[IntroL[IntroCounter+i].Decode]--;
            IntroL[IntroCounter+i].ScrambleData[IntroL[IntroCounter+i].Decode]--;
          } else {
            IntroL[IntroCounter+i].Decode++;
          }
          HoldASec[i] = 0;
        } else {
          HoldASec[i] = 1;
        }
        Font.glPrint(AC.GetHalfW()-strlen(IntroL[IntroCounter+i].Text)*12/2,AC.GetHalfH()-24+i*24,IntroL[IntroCounter+i].Text,i);

      }
      GFX.SetLightning(0);
      break;

    case MENU_MAIN:

      GFX.SetLightning(1);
      Font.glPrint((AC.GetHalfW())-(strlen(Menu.GetItemName(1))*12/2),AC.GetHalfH()-16,Menu.GetItemName(1),2);
      Font.glPrint((AC.GetHalfW())-(strlen(Menu.GetItemName(0))*12/2),AC.GetHalfH(),Menu.GetItemName(0),1); // Sel
      Font.glPrint((AC.GetHalfW())-(strlen(Menu.GetItemName(2))*12/2),AC.GetHalfH()+16,Menu.GetItemName(2),2);
      GFX.SetLightning(0);

      if( Menu.decode < Menu.lenwelc ){
        if( Menu.scrm[Menu.decode] > 0 ){
          Menu.welc[Menu.decode]--;
          Menu.scrm[Menu.decode]--;
        } else {
          Menu.decode++;
        }
        const char *decm = "Decoding Message";
        if( Menu.ShowInfo ) Font.glPrint((AC.GetHalfW())-(strlen(decm)*12/2),AC.GetHalfH()/2-32,decm,0);
      }
      Font.glPrint((AC.GetHalfW())-(strlen(Menu.welc)*12/2),AC.GetHalfH()/2,Menu.welc,1);

      
      break;
    case MENU_LOADING_MAP:

      Font.glPrint((AC.GetHalfW())-(strlen(Menu.GetMenuName())*12/2),AC.GetHalfH()/4,Menu.GetMenuName(),0);
      Font.glPrint((AC.GetHalfW())-(strlen(Menu.GetOptinalInfo())*12/2),AC.GetHalfH()-8,Menu.GetOptinalInfo(),1);

      break;
    case MENU_SETTINGS_MAIN:

      Font.glPrint((AC.GetHalfW())-(strlen(Menu.GetMenuName())*12/2),AC.GetHalfH()/4,Menu.GetMenuName(),1);
      GFX.SetLightning(1);
      for( i = 0; i < Menu.GetNumItems(); i++ ){
        Font.glPrint((AC.GetHalfW())-(strlen(Menu.GetFreeItemName(i))*12/2),AC.GetHalfH()-(16*Menu.GetItem())+16*i,Menu.GetFreeItemName(i),!Menu.IsSelected(i)+1);
      }
      GFX.SetLightning(0);

      break;
    case MENU_SETTINGS_PROFILE:
            
      

      break;
    case MENU_SETTINGS_CONTROLS:

      Font.glPrint((AC.GetHalfW())-(strlen(Menu.GetMenuName())*12/2),AC.GetHalfH()/4,Menu.GetMenuName(),1);
      GFX.SetLightning(1);
      for( i = 0; i < Menu.GetNumItems(); i++ ){
        Font.glPrint((AC.GetHalfW())-(strlen(Menu.GetFreeItemName(i))*12/2),AC.GetHalfH()-(16*Menu.GetItem())+16*i,Menu.GetFreeItemName(i),!Menu.IsSelected(i)+1);
      }
      GFX.SetLightning(0);

      break;
    case MENU_SETTINGS_CHECK_BINDS:

      Font.glPrint((AC.GetHalfW())-(strlen(Menu.GetMenuName())*12/2),AC.GetHalfH()/4,Menu.GetMenuName(),1);
      GFX.SetLightning(1);
      for( i = 0; i < CTRL_NUM_OF_CONTROL_KEYS; i++ ){
        Font.glPrint((AC.GetHalfW())-(strlen(Control.GetBoundString(i))*12/2),AC.GetHalfH()-(12*CTRL_NUM_OF_CONTROL_KEYS/2)+16*i,Control.GetBoundString(i),0);
      }
      GFX.SetLightning(0);

      break;
    case MENU_SETTINGS_SET_BINDS:

      Font.glPrint((AC.GetHalfW())-(strlen(Menu.GetMenuName())*12/2),AC.GetHalfH()/4,Menu.GetMenuName(),1);
      GFX.SetLightning(1);
      for( i = 0; i < Menu.GetNumItems(); i++ ){
        Font.glPrint((AC.GetHalfW())-(strlen(Control.GetBoundString(i))*12/2),AC.GetHalfH()-(16*Menu.GetItem())+16*i,Control.GetBoundString(i),!Menu.IsSelected(i)+1);
      }
      GFX.SetLightning(0);

      break;
    case MENU_SETTINGS_SFX:

      Font.glPrint((AC.GetHalfW())-(strlen(Menu.GetMenuName())*12/2),AC.GetHalfH()/4,Menu.GetMenuName(),1);
      GFX.SetLightning(1);
      for( i = 0; i < Menu.GetNumItems(); i++ ){
        Font.glPrint((AC.GetHalfW())-(strlen(Menu.GetFreeItemName(i))*12/2),AC.GetHalfH()-(16*Menu.GetItem())+16*i,Menu.GetFreeItemName(i),!Menu.IsSelected(i)+1);
      }
      GFX.SetLightning(0);
      Font.glPrint((AC.GetHalfW())-(strlen(Menu.GetOptinalInfo())*12/2),AC.GetScreenHeight()-64,Menu.GetOptinalInfo(),0);

      break;
    case MENU_SETTINGS_GFX:

      Font.glPrint((AC.GetHalfW())-(strlen(Menu.GetMenuName())*12/2),AC.GetHalfH()/4,Menu.GetMenuName(),1);
      GFX.SetLightning(1);
      for( i = 0; i < Menu.GetNumItems(); i++ ){
        Font.glPrint((AC.GetHalfW())-(strlen(Menu.GetFreeItemName(i))*12/2),AC.GetHalfH()-(16*Menu.GetItem())+16*i,Menu.GetFreeItemName(i),!Menu.IsSelected(i)+1);
      }
      GFX.SetLightning(0);
      Font.glPrint((AC.GetHalfW())-(strlen(Menu.GetOptinalInfo())*12/2),AC.GetScreenHeight()-64,Menu.GetOptinalInfo(),0);

      break;
    default:
      break;
  }

  // Slow stuff down a tad.
  usleep(AC.GetProgramHoldTime());

  // Get FPS
  AC.FPSc++;
  time(&end);
  AC.dif = AC.dif + difftime (end,start);
  if( AC.dif >= 1 ){
    sprintf(AC.FPS,"Running this screen at %i FPS",AC.FPSc);
    AC.FPSc = 0;
    AC.dif = 0;

    Menu.ShowInfo = !Menu.ShowInfo;
    Menu.InfoDelay++;
    if( Menu.InfoDelay > Menu.InfoDelayStop ){
      Menu.InfoDelay = 0;
      Starfield.SetFacing(int(rand()%360));
    }

    if( HoldASec[0] == HoldASec[1] && HoldASec[0] == 1 ){
      IntroWait++;
      if( IntroWait > 2 ){
        HoldASec[0] = 0;
        HoldASec[1] = 0;
        IntroCounter+=2;
        IntroWait = 0;
        if( IntroCounter > (INTROLINES - 2) ){
          IntroCounter = 0;
          Menu.SetMenuSelection(MENU_MAIN);
        }
      }
    }
  }
}






/* The main drawing function. */
void DrawGLScene()
{

  glClear(GL_COLOR_BUFFER_BIT);
  glLoadIdentity(); // Reset The View

  // Enable lighting for the stars
  GFX.SetLightning(1);
    DrawStarfield();
    Starfield.StarPhysics();
  GFX.SetLightning(0);
  
  switch( AC.GetSpecifiedLoop() ){
    case LOOP_MAIN_MENU:
      DrawMenu();
      break;
    case LOOP_GAME:
      DrawMainGame();
      break;
    case LOOP_IN_GAME_MENU:
      break;
    case LOOP_TERMINAL:
      DrawTerminal();
      break;
    default:
      DrawMenu();
      break;
  }
  if( AC.QuitConfirmed() )
    QuitAC();

  // since this is double buffered, swap the buffers to display what just got drawn.
  glutSwapBuffers();

  

}

/* The function called when our window is resized (which shouldn't happen, because we're fullscreen) */
void ReSizeGLScene(int Width, int Height)
{
  if (Height==0)				// Prevent A Divide By Zero If The Window Is Too Small
	  Height=1;

  glViewport(0, 0, Width, Height);		// Reset The Current Viewport And Perspective Transformation

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

//  glMatrixMode(GL_MODELVIEW);
}




































































int main(int argc, char **argv) 
{

  SYSTEM.NewSection("Writing Game Core Information");

  
  Profile.LoadProfileSettings();

  AC.SetLinScale(0);
  
//  AC.SetResolution(2,1);
//  AC.SetLinScale(0);
//
//  #ifdef USEFUL
//    AC.SetFullscreen();
//  #else
//    AC.SetWindowed();
//  #endif
  AC.PrintInformation();

  /* Initialize GLUT state - glut will take any command line arguments that pertain to it or 
     X Windows - look at its documentation at http://reality.sgi.com/mjk/spec3/spec3.html */  
  glutInit(&argc, argv);  

  glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_ALPHA);

  glutInitWindowSize(AC.GetGameWidth(), AC.GetGameHeight());

  if( AC.GetFullscreen() ){
    glutInitWindowPosition(0, 0);
  } else {
    glutInitWindowPosition(200, 100);
  }

  Glut_Window = glutCreateWindow(AC.GetWindowTitle());

  glutDisplayFunc(&DrawGLScene);  

  if( AC.GetFullscreen() == 1 ){
    glutFullScreen();
  }

  // Even if there are no events, redraw our gl scene.
  glutIdleFunc(&DrawGLScene);

  // Register the function called when our window is resized.
  glutReshapeFunc(&ReSizeGLScene);

  // Register the function called when the keyboard is pressed.
  glutIgnoreKeyRepeat(1);
	glutKeyboardFunc(&keyPressed);
	glutKeyboardUpFunc(&keyReleased);
  
//  glutKeyboardFunc(&keyPressed);




  SetupGameSettings();

  /* Initialize our window. */
  GFX.InitGL(AC.GetScreenWidth(), AC.GetScreenHeight());

  AC.ConfirmInitiated();
  
  /* Start Event Processing Engine */  
  glutMainLoop();  

  return 1;
}

